/**
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <NetworkManager.h>
#include <dbus/dbus-glib.h>
#include <libxfce4util/libxfce4util.h>

#include "ltop-login-box.h"
#include "main.h"

static GtkListStore *targets_store;
static GtkWidget *detail_label;
static GtkWidget *login;
static GtkWidget *login_btn;
static GtkWidget *go_btn;
//static GtkWidget *go_upgrade_btn;
static GdkPixbuf *image_pxb = NULL, *config_pxb = NULL;
static GtkWidget *p_dialog;
static GtkWidget *p_bar;

static gchar *config_message = NULL;

G_LOCK_DEFINE_STATIC (status);
static volatile int status;

#define DETAIL_LABEL_DEFAULT "Select a task..."
#define GO_BUTTON_DEFAULT "Apply"

static gpointer
do_config_thread (LtopLoginInfoData *info)
{
    gboolean ret;

    ret = ltop_login_box_apply_config (info->id, info->version,
                                ltop_login_box_get_email (LTOP_LOGIN_BOX (login)),
                                ltop_login_box_get_invoice (LTOP_LOGIN_BOX (login)),
                                &config_message);
    G_LOCK (status);
    status = ret ? 0 : 1;
    G_UNLOCK (status);

    return NULL;
}

static gpointer
do_show_progress (GtkWidget *pbar)
{
    GtkWidget *label;
    GtkWidget *textview;
    GtkWidget *scw;

    while (1)
    {
        int s;
        G_LOCK (status);
        s = status;
        G_UNLOCK (status);

        if (s != 2)
            break;

        gdk_threads_enter ();
        gtk_progress_bar_pulse (GTK_PROGRESS_BAR (p_bar));
        gdk_threads_leave ();

        g_usleep (50000);
    }
    gtk_widget_destroy (p_dialog);

    G_LOCK (status);
    gdk_threads_enter ();

    p_dialog = gtk_dialog_new_with_buttons (_("Done"), NULL, GTK_DIALOG_MODAL,
                                            GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
    gtk_dialog_set_has_separator (GTK_DIALOG (p_dialog), FALSE);
    gtk_window_set_default_size (GTK_WINDOW (p_dialog), 460, 300);
    gtk_container_set_border_width (GTK_CONTAINER (p_dialog), 5);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
                          (status == 0) ? _("<b>The configuration is applied.\n</b>")
                                        : _("<b>A problem occured.\n</b>"));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_widget_show (label);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (p_dialog)->vbox), label, FALSE, FALSE, 2);

    scw = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (p_dialog)->vbox), scw, TRUE, TRUE, 2);

    textview = gtk_text_view_new ();
    gtk_container_add (GTK_CONTAINER (scw), textview);
    gtk_text_view_set_editable (GTK_TEXT_VIEW (textview), FALSE);
    gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview), GTK_WRAP_WORD);
    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview), FALSE);
    gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview)),
                              config_message, -1);
    g_free (config_message);
    config_message = NULL;
    gtk_widget_show_all (scw);

    gtk_dialog_run (GTK_DIALOG (p_dialog));
    gtk_widget_destroy (p_dialog);
    gdk_threads_leave ();
    G_UNLOCK (status);

    return NULL;
}

static gboolean
on_dialog_delete_event (GtkWidget *d, GdkEvent *e, gpointer data)
{
    return TRUE;
}

static void
populate_store (LtopLoginBox *lb)
{
    GList *lp;
    GtkTreeIter iter;
    static LtopLoginInfo *infos = NULL;

    if (infos)
    {
        ltop_login_info_free (infos);
        infos = NULL;
    }

    gtk_list_store_clear (targets_store);

    if (!ltop_login_box_check_connected ())
    {
        gtk_label_set_markup (GTK_LABEL (detail_label),
                            _("<span fgcolor=\"red\">Disconnected.</span> "
                              "Please check your Internet connection."));
        return;
    }

    if (!image_pxb)
    {
        GtkIconTheme *theme;
        theme = gtk_icon_theme_get_default ();
        image_pxb = gtk_icon_theme_load_icon (theme, "system-software-install",
                                              24, 0, NULL);
        config_pxb = gtk_icon_theme_load_icon (theme, "gnome-control-center",
                                              24, 0, NULL);
    }

    gtk_list_store_clear (targets_store);

    infos = ltop_login_box_login (lb);

    /* display the update button only if OS images are available */
    /*
    if (g_list_length (infos->images) > 0)
        gtk_widget_set_sensitive (go_upgrade_btn, TRUE);
    */

    for (lp=infos->images;lp;lp=lp->next)
    {
        LtopLoginInfoData *data = (LtopLoginInfoData *) lp->data;
        gtk_list_store_append (targets_store, &iter);
        gtk_list_store_set (targets_store, &iter,
                            0, image_pxb,
                            1, data->text,
                            2, (gpointer) data,
                            -1);
    }

    for (lp=infos->configs;lp;lp=lp->next)
    {
        LtopLoginInfoData *data = (LtopLoginInfoData *) lp->data;
        gtk_list_store_append (targets_store, &iter);
        gtk_list_store_set (targets_store, &iter,
                            0, config_pxb,
                            1, data->text,
                            2, (gpointer) data,
                            -1);
    }
}

/*
static void
on_go_upgrade_btn_clicked (GtkWidget *btn, gpointer data)
{
    gchar *cmd;
    const gchar *email;
    const gchar *invoice;

    email = ltop_login_box_get_email (LTOP_LOGIN_BOX (login));
    invoice = ltop_login_box_get_invoice (LTOP_LOGIN_BOX (login));

    cmd = g_strdup_printf ("lwebinstall -u -o %s -e %s",
                           invoice, email);
    xfce_exec (cmd, FALSE, FALSE, NULL);
    g_free (cmd);
}
*/

static void
on_go_btn_clicked (GtkWidget *btn, gpointer treeview)
{
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    GtkTreeIter iter;
    LtopLoginInfoData *info;

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
    if (!gtk_tree_selection_get_selected (selection, &model, &iter))
        return;

    gtk_tree_model_get (model, &iter, 2, &info, -1);
    if (info->type == LTOP_LOGIN_INFO_DATA_TYPE_IMAGE)
    {
        gchar *cmd;
        const gchar *email;
        const gchar *invoice;

        email = ltop_login_box_get_email (LTOP_LOGIN_BOX (login));
        invoice = ltop_login_box_get_invoice (LTOP_LOGIN_BOX (login));

        cmd = g_strdup_printf ("lwebinstall -o %s -e %s -c %s",
                               invoice, email, info->id);
        xfce_exec (cmd, FALSE, FALSE, NULL);
        g_free (cmd);
    }
    else
    {
        GtkWidget *label;

        status = 2;

        p_dialog = gtk_dialog_new_with_buttons (_("Applying configuration"),
                                         NULL,
                                         GTK_DIALOG_MODAL,
                                         NULL);
        g_signal_connect (G_OBJECT (p_dialog), "delete-event",
                          G_CALLBACK (on_dialog_delete_event), NULL);
        gtk_dialog_set_has_separator (GTK_DIALOG (p_dialog), FALSE);
        label = gtk_label_new (_("Please wait while executing the request..."));
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (p_dialog)->vbox), label,
                            FALSE, FALSE, 2);
        gtk_widget_show (label);

        p_bar = gtk_progress_bar_new ();
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (p_dialog)->vbox), p_bar,
                            FALSE, FALSE, 2);
        gtk_widget_show (p_bar);

        gtk_widget_show_all (p_dialog);

        g_thread_create ((GThreadFunc) do_show_progress, NULL, TRUE, NULL);
        g_thread_create ((GThreadFunc) do_config_thread, info, FALSE, NULL);
    }
}

static void
on_treeview_cursor_changed (GtkWidget *treeview, gpointer data)
{
    GtkTreeSelection *selection;
    GtkTreeModel *model;
    GtkTreeIter iter;
    gchar *markup;
    LtopLoginInfoData *info;

    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
    if (!gtk_tree_selection_get_selected (selection, &model, &iter))
    {
        gtk_label_set_text (GTK_LABEL (detail_label), _(DETAIL_LABEL_DEFAULT));
        gtk_button_set_label (GTK_BUTTON (go_btn), _(GO_BUTTON_DEFAULT));
        gtk_widget_set_sensitive (go_btn, FALSE);
        return;
    }

    gtk_tree_model_get (model, &iter, 2, &info, -1);

    gtk_widget_set_sensitive (go_btn, TRUE);

    if (info->type == LTOP_LOGIN_INFO_DATA_TYPE_IMAGE)
    {
        gtk_button_set_label (GTK_BUTTON (go_btn),
                    _("Install"));
        markup = g_strdup_printf ("<b>Linutop OS download:</b>\n%s", info->text);
        gtk_label_set_markup (GTK_LABEL (detail_label), markup);
    }
    else
    {
        gtk_button_set_label (GTK_BUTTON (go_btn),
                    _("Apply"));
        if (info->desc)
            markup = g_strdup_printf ("<b>System configuration:</b>\n%s",
                                      info->desc);
        else
            markup = g_strdup_printf ("<b>System configuration:</b>\n%s",
                                      info->text);
        gtk_label_set_markup (GTK_LABEL (detail_label), markup);
    }

    g_free (markup);
}

static void
on_login_btn_clicked (GtkWidget *btn, gpointer data)
{
    populate_store ((LtopLoginBox *) data);
}

static void
on_url_clicked (GtkWidget *w, gpointer data)
{
    run_user_cmd ("firefox http://linutop.com/shop", TRUE);
}

static gboolean
after_nm_connected (gpointer data)
{
    populate_store ((LtopLoginBox *) data);
    gtk_label_set_text (GTK_LABEL (detail_label), _(DETAIL_LABEL_DEFAULT));

    return FALSE;
}

static void
nm_state_changed (DBusGProxy *proxy,
                  guint state,
                  gpointer data)
{
    if (state == NM_STATE_CONNECTED)
    {
        g_timeout_add_seconds (1, after_nm_connected, data);
    }
    else if (state == NM_STATE_DISCONNECTED)
    {
        populate_store ((LtopLoginBox *) data);
    }
}

static void
setup_dbus_service (gpointer data)
{
    DBusGConnection *connection;
    DBusGProxy *proxy;

    connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, NULL);
    proxy = dbus_g_proxy_new_for_name (connection,
                "org.freedesktop.NetworkManager",
                "/org/freedesktop/NetworkManager",
                "org.freedesktop.NetworkManager");

    dbus_g_proxy_add_signal (proxy, "StateChanged",
                             G_TYPE_UINT, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal (proxy, "StateChanged",
                                 G_CALLBACK (nm_state_changed), data, NULL);
}

GtkWidget *
get_service_widget ()
{
    GtkWidget *vbox, *list_frame, *login_frame;
    GtkWidget *box, *hbox, *label;
    GtkWidget *lvbox, *lhbox;
    GtkWidget *treeview;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkWidget *img;
    GtkWidget *sc;

    vbox = gtk_vbox_new (FALSE, 0);

    /* Login */
    lvbox = gtk_vbox_new (FALSE, 0);
    lhbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (lvbox), lhbox, FALSE, FALSE, 0);

    login = ltop_login_box_new (NULL, NULL);
    login_btn = gtk_button_new_with_label (_("Login"));
    img = gtk_image_new_from_icon_name ("internet-web-browser",
                                        GTK_ICON_SIZE_BUTTON);
    gtk_button_set_image (GTK_BUTTON (login_btn), img);
    g_signal_connect (G_OBJECT (login_btn), "clicked",
                      G_CALLBACK (on_login_btn_clicked), login);
    box = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_end (GTK_BOX (box), login_btn, FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (lhbox), login, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (lhbox), box, FALSE, FALSE, 2);

    label = sexy_url_label_new ();
    sexy_url_label_set_markup (SEXY_URL_LABEL (label),
        _("<b>Purchase an installation or configuration</b> on "
          "<a href=\"http://www.linutop.com/shop\">"
          "the Linutop shop</a>."));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    g_signal_connect (G_OBJECT (label), "url_activated",
        G_CALLBACK (on_url_clicked), NULL);
    gtk_box_pack_start (GTK_BOX (lvbox), label, FALSE, FALSE, 6);

    login_frame = xfce_framebox_new (_("Customer login"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (login_frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (login_frame), lvbox);

    /* listing */
    box = gtk_vbox_new (FALSE, 0);
    sc = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sc),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);

    targets_store = gtk_list_store_new (3, GDK_TYPE_PIXBUF,
                                           G_TYPE_STRING,
                                           G_TYPE_POINTER);
    treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (targets_store));
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
    g_signal_connect (G_OBJECT (treeview), "cursor-changed",
                      G_CALLBACK (on_treeview_cursor_changed), detail_label);

    /* icon column */
    renderer = gtk_cell_renderer_pixbuf_new ();
    column = gtk_tree_view_column_new_with_attributes ("",
                                              renderer,
                                              "pixbuf", 0,
                                              NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

    /* small desc */
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("",
                                                       renderer,
                                                       "text", 1,
                                                       NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

    gtk_container_add (GTK_CONTAINER (sc), treeview);
    gtk_box_pack_start (GTK_BOX (box), sc, FALSE, FALSE, 2);

    hbox = gtk_hbox_new (FALSE, 2);
    detail_label = gtk_label_new (_(DETAIL_LABEL_DEFAULT));
    gtk_misc_set_alignment (GTK_MISC (detail_label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (hbox), detail_label, TRUE, TRUE, 2);

    lvbox = gtk_vbox_new (FALSE, 0);
    go_btn = gtk_button_new_with_label (_(GO_BUTTON_DEFAULT));
    gtk_button_set_image (GTK_BUTTON (go_btn),
                          gtk_image_new_from_stock (GTK_STOCK_APPLY,
                                                    GTK_ICON_SIZE_BUTTON));
    gtk_widget_set_sensitive (go_btn, FALSE);
    g_signal_connect (G_OBJECT (go_btn), "clicked",
                      G_CALLBACK (on_go_btn_clicked), treeview);
    gtk_box_pack_end (GTK_BOX (lvbox), go_btn, FALSE, FALSE, 0);
    gtk_box_pack_end (GTK_BOX (hbox), lvbox, FALSE, FALSE, 2);

/*
    go_upgrade_btn = gtk_button_new_with_label (_("Upgrade"));
    gtk_button_set_image (GTK_BUTTON (go_upgrade_btn),
                          gtk_image_new_from_icon_name (
                            "software-update-available",
                            GTK_ICON_SIZE_BUTTON));
    gtk_widget_set_sensitive (go_upgrade_btn, FALSE);
    g_signal_connect (G_OBJECT (go_upgrade_btn), "clicked",
                      G_CALLBACK (on_go_upgrade_btn_clicked), treeview);
    gtk_box_pack_end (GTK_BOX (box), go_upgrade_btn, FALSE, FALSE, 2);
*/

    gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 2);

    list_frame = xfce_framebox_new (_("Installation and configuration"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (list_frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (list_frame), box);

    gtk_box_pack_start (GTK_BOX (vbox), login_frame, FALSE, FALSE, 4);
    gtk_box_pack_start (GTK_BOX (vbox), list_frame, TRUE, TRUE, 4);

    populate_store (LTOP_LOGIN_BOX (login));
    setup_dbus_service (login);

    return vbox;
}
