/**
 * Copyright (C) 2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"

static void
on_shutdown (GObject *btn, gpointer data)
{
    GtkWidget *d;
    gint rlevel = 6;
    const gchar *msg;

    rlevel = GPOINTER_TO_INT (g_object_get_data (btn, "role"));
    if (rlevel == 6)
        msg = _("Do you really want to reboot the system?");
    else
        msg = _("Do you really want to halt the system?");

    d = gtk_message_dialog_new (NULL,
                                GTK_DIALOG_MODAL,
                                GTK_MESSAGE_QUESTION,
                                GTK_BUTTONS_YES_NO,
                                msg,
                                NULL);
    if (gtk_dialog_run (GTK_DIALOG (d)) != GTK_RESPONSE_YES)
    {
        gtk_widget_destroy (d);
        return;
    }

    system ((rlevel == 6) ? "reboot" : "halt");
}

GtkWidget *
get_tools_widget ()
{
    GtkWidget *vbox, *box;
    GtkWidget *frame;
    GtkWidget *halt_btn;
    GtkWidget *reboot_btn;

    vbox = gtk_vbox_new (FALSE, 0);

    box = gtk_vbox_new (FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
        new_run_cmd_button ("gnome-terminal", "Terminal",
                            "gnome-terminal", FALSE),
        FALSE, FALSE, 2);

    reboot_btn = gtk_button_new ();
    g_object_set_data (G_OBJECT (reboot_btn), "role", GINT_TO_POINTER (6));
    gtk_button_set_image (GTK_BUTTON (reboot_btn),
                          get_image_from_str ("view-refresh"));
    gtk_button_set_label (GTK_BUTTON (reboot_btn), "Reboot");
    gtk_button_set_alignment (GTK_BUTTON (reboot_btn), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (box), reboot_btn, FALSE, FALSE, 2);

    halt_btn = gtk_button_new ();
    g_object_set_data (G_OBJECT (halt_btn), "role", GINT_TO_POINTER (0));
    gtk_button_set_image (GTK_BUTTON (halt_btn),
                          get_image_from_str ("system-shutdown"));
    gtk_button_set_label (GTK_BUTTON (halt_btn), "Halt");
    gtk_button_set_alignment (GTK_BUTTON (halt_btn), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (box), halt_btn, FALSE, FALSE, 2);

    g_signal_connect (G_OBJECT (reboot_btn), "clicked",
                      G_CALLBACK (on_shutdown), NULL);
    g_signal_connect (G_OBJECT (halt_btn), "clicked",
                      G_CALLBACK (on_shutdown), NULL);

    frame = xfce_framebox_new (_("Tools"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), box);

    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);

    return vbox;
}
